<?php

/**
 * Internal toolset for Views remote pager module.
 */

class VRP {

  // Internal caching.
  private static $navigation;

  // Related to original view.
  private static $view, $nid, $position;

  /**
   * Adds a vrp query to the url of any node which is result of a VRP view.
   *
   * @param object $node
   *   A node object about to be linked.
   *
   * @return bool|array
   *   If VRP data was found in the query,
   */
  public static function getViewParams($node = NULL) {

    $ret = FALSE;
    if (
      (is_object($node))
      &&
      ($view =@ $node->view)
      &&
      (is_object($view))
      &&
      // Check if the display is vrp-enabled.
      ($display =@ $view->display[$view->current_display])
      &&
      (!empty($display->display_options['vrp_enabled']))
      &&
      // Check if this actually is a paged result.
      (isset($view->row_index))
      &&
      // Make sure a views ID is provided.
      ($source_view_id =@ $view->name)
      &&
      // Make sure a views ID is provided.
      ($source_display_id =@ $view->current_display)
    ) {
      // Prepare query parameters.
      $original_query = $_GET;
      unset($original_query['q']);
      $original_query = drupal_http_build_query($original_query);
      $params = array(
        'v' => $source_view_id,
        'd' => $source_display_id,
        'q' => $original_query,
        'p' => current_path(),
      );
      // Add views internal params. These may not live in the query,
      // but reflect the default settings for exposed filters.
      if (($views_query =@ $view->exposed_raw_input) && is_array($views_query)) {
        $params['vq'] = urlencode(drupal_http_build_query($views_query));
      }

      $ret = array('vrp' => $params);
    }
    return $ret;
  }

  /**
   * Builds a navigation from the current global request, if it indicates so.
   *
   * Checks the request data for existing views data. If such data is found,
   * the corresponding view is looked up, executed and the position of the
   * current node in the result set is evaluated.
   *
   * @param bool $force
   *   Rebuild, even if previously cached.
   *
   * @return array|bool
   *
   */
  public static function getNavigationData($force = FALSE) {

    if (!isset(self::$navigation) || $force) {
      $ret = FALSE;
      $originalQuery = $_GET;
      // Remove original path request.
      unset($originalQuery['q']);

      $q = explode('/', current_path());
      // Will currently only work on node pages.
      if (is_array($q) && array_shift($q) == 'node') {
        if ($nid = array_shift($q)) {
          self::$nid = $nid;
          if ($query =@ $originalQuery['vrp']) {
            if (($view_id =@ $query['v']) && ($display_id =@ $query['d'])){
              if ($view =@ views_get_view($view_id)) {
                if ($view->set_display($display_id)) {
                  $view->pre_execute();
                  $view-> execute();
                  if (!empty($view->result)) {
                    self::$view = $view;
                    // $view->result now contains all rows, so we can build a pager.

                    // @todo determine position from actual views result!
                    $current_position = self::getPosition();

                    $max = count($view->result) - 1;
                    $ret = array();
                    if ($current_position > 0) {
                      $ret += array(
                        'first' => 0,
                        'previous' => $current_position - 1,
                      );
                    }
                    if ($current_position < $max) {
                      $ret += array(
                        'next' => $current_position + 1,
                        'last' => $max,
                      );
                    }

                    // @todo create "up" link (=back to view and the page corresponding with the current node,
                    // i.e. e.g. "/front?page=13#node-nid") <= mind infinite scroll!

                    // Build navigation links; respect original query.
                    foreach (array_keys($ret) as $key) {
                      $pos = $ret[$key];
                      // Determine node nid from views result
                      // (is always present in "content" row style).
                      $nid = $view->result[$pos]->nid;
                      $ret[$key] = url("node/$nid", array('query' => $originalQuery));
                    };
                  }
                }
              }
            }
          }
        }
      }
      self::$navigation = $ret;
    }
    return self::$navigation;
  }

  /**
   * Adds "<link rel=...>" tags depending on the actual navigation state.
   */
  public static function injectRelations() {
    if ($nav = self::getNavigationData()) {
      foreach ($nav as $type => $href) {
        drupal_add_html_head(array(
          '#tag' => 'link',
          '#attributes' => array( // Set up an array of attributes inside the tag
            'rel' => $type,
            'href' => $href,
          ),
        ), "vrp_nav_$type");
      }
    }
  }

  /**
   * Determines the position of the current node in the view result array.
   *
   * @return int|bool
   *   The position of the current node in the entire views result
   *   FALSE, if there is no position (the node is not or no longer
   *   part of the provided view).
   *
   * CAUTION:
   *   "0" may be a valid position result. You may want to apply
   *   strict comparison on the return value ("===").
   */
  private static function getPosition() {
    if (!isset(self::$position)) {
      if (is_object(self::$view) && !empty(self::$view->result) && isset(self::$nid)) {
        foreach (self::$view->result as $pos => $data) {
          if ($data->nid == self::$nid) {
            // Will ignore non-distinct queries by design, so multiple positions
            // will not work.
            self::$position = $pos;
            break;
          }
        }
      }
    }
    return self::$position;
  }

}
